/* SaveWorkSheets
 * das Programm speichert alle Arbeitsblatter einer MegaCAD-Zeichnung 
 * im Parameter angegeben Format ab. Als Dateiname wird der Name des
 * 3D-Modells, erweitert um einen Bindestrich und den Arbeitsblattnamen 
 * verwendet. <ModelName-ArbeitsblattName.Format>	
 *
 * @Version: 0.1																	
 * @Autor: JonCarpenter	
 * @Copyright: (C) 2014-02-02 
 * @Lizenz: GPL	 
 *
 */
 
#pragma StackSize 300000
#pragma HeapMin 50000

#include "std.h"
#include "megatyp.h"
#include "megacad.h"

short 	fh1 = 0;
short 	fh2 = 0;
	
char	line[1024];
char	command[1024];
char	fileName[1024];
char	srcPath[1024];
char	dstPath[1024];
char	srcTyp[8];
char	dstTyp[8];


static void SetDlg(t_dlgdta *t, char *text, uchar typ, void *val)
{
    strcpy(&t->text[0],text);
    t->type = typ;
    t->val = val;
    return;
}

	
// sucht das letzte vorkommen des Tokens "token" im String "str"
// Gibt die Anzahl der Zeichen von Stringanfang bis zum lezten Vorkommen zurück.
short findLastToken(char str[], char token)
{
	short lastMatchPos = 0;
	short currentPos = 0;
	while(str[currentPos] != '\0')
	{
		if(str[currentPos] == token)
		{
			lastMatchPos = currentPos;
		}
		currentPos++;
	}
	return lastMatchPos;
}

// wir für jedes gefundene Arbeitsblatt aufgerufen
// speichert das Arbeitsblatt im angegeben Format
short SaveWorkingSheet(void *arg)
{
	short n = 0;
	short m = 0;
	char *name = NULL;
	t_name workSheet;
	
	//Namen der aktuell geladenen Datei ermitteln
	GetCurrentFile(fileName);	
    
	//Dateinamen und Endung abschneiden 
	n = findLastToken(fileName, '\\');
	name = &fileName[n+1];
	m = findLastToken(name, '.');
	name[m] = '\0';
	
	//Arbeitsblattdaten ermitteln
	GetWorkingSheet(&workSheet);
	
	//Komandozeile zusammenstellen
	sprintf(command, "%s\"%s%s-%s%s\"", "PrtSavAs ", dstPath, name, workSheet.name, dstTyp);
	ML_Printf(fh2, "  found Sheet %s -> %s\n", workSheet.name, command);
	MegaCMD(command);

	return(CONTINUE_LOOP);
}

// Hauptprogramm
short main(char *filename, char *args)
{
	int 	count  = 0;
	short 	n = 0;
	short 	m = 0;
	short	subDir = 0;
	short	srcFormat = 0;
	short	dstFormat = 0;
	short	mret = 0;	//Rückgabewert des Dialogelemnts
	
	t_dlgdta tg[12]; 	//Array für Dialogelemente

	
	T_EDTA *edta;
	edta = GetVarPtr();
	
	//festlegen der Dialogfeldinhalte
	SetDlg(&tg[0],"PRT",					DLG_RAD, &dstFormat);
	SetDlg(&tg[1],"MAC",					DLG_RAD, &dstFormat);
	SetDlg(&tg[2],"DXF",					DLG_RAD, &dstFormat);
	SetDlg(&tg[3],"DWG",					DLG_RAD, &dstFormat);
	SetDlg(&tg[4],"SAT",					DLG_RAD, &dstFormat);
	SetDlg(&tg[5],"IGS",					DLG_RAD, &dstFormat);
	SetDlg(&tg[6],"STP",					DLG_RAD, &dstFormat);
	SetDlg(&tg[7],"STL",					DLG_RAD, &dstFormat);
	//SetDlg(&tg[8],"PDF",					DLG_RAD, &dstFormat);
	SetDlg(&tg[8],"Quellverzeichnis", 		DLG_BUT, NULL);
	SetDlg(&tg[9],"Incl. Unterverzeichnisse", DLG_CHK, &subDir);
    //SetDlg(&tg[10],"Zielverzeichnis", 		DLG_BUT, NULL);
    SetDlg(&tg[10],"Umwandeln", 			DLG_BUT, NULL);
	SetDlg(&tg[11],"Abbruch", 				DLG_DEF, NULL);
   

	// aktuellen Pfad und Dateinamen ermitteln
	GetCurrentFile(fileName);	
    	
	// Pfad + Dateinamen, Dateinamen abschneiden und "*.*" anhängen
	sprintf(srcPath, "%s", fileName);
	n = findLastToken(srcPath, '\\');
	srcPath[n+1] = '\0';
	//sprintf(srcPath, "%s%s", srcPath, "*.*");
	
	// Pfad + Dateinamen, Dateinamen abschneiden und "*.*" anhängen
	sprintf(dstPath, "%s", fileName);
	n = findLastToken(dstPath, '\\');
	dstPath[n+1] = '\0';	
	//sprintf(dstPath, "%s%s", dstPath, "*.*");

	//Voreinstellungen für Dialog festlegen
	subDir = FALSE;
	srcFormat = 0;
	dstFormat = 2;
	
	do
	{
		//Dialog aufrufen
		mret = DlgInput(11,"Export WorkSheets",tg);

		// Dateiendungen Zieldateien erstellen
		switch(dstFormat)
		{
			case  0: sprintf(dstTyp, "%s", ".PRT"); break;
			case  1: sprintf(dstTyp, "%s", ".MAC"); break;
			case  2: sprintf(dstTyp, "%s", ".DXF"); break;
			case  3: sprintf(dstTyp, "%s", ".DWG"); break;
			case  4: sprintf(dstTyp, "%s", ".SAT"); break;
			case  5: sprintf(dstTyp, "%s", ".IGS"); break;
			case  6: sprintf(dstTyp, "%s", ".STP"); break;
			case  7: sprintf(dstTyp, "%s", ".STL"); break;
			case  8: sprintf(dstTyp, "%s", ".PDF"); break;
			default: sprintf(dstTyp, "%s", ".DXF"); break;
		}
		
		switch(mret-1)
		{
		case 8:
			// Datei (Verzeichnis) auswählen
			sprintf(srcPath, "%s%s", dstPath, "*.*");
			if( GetFilename("Eine Datei aus dem Startordner auswählen", srcPath) == TRUE )
			{
				// Dateinamen abschneiden und Endung finden
				n = findLastToken(srcPath, '\\');
				m = findLastToken(srcPath, '.');
				sprintf(srcTyp, "%s", &srcPath[m]);
				srcPath[n+1] = '\0';
				sprintf(dstPath, "%s", srcPath);
				//Message(srcPath, srcTyp, NULL,NULL,"OK",3);
			}		
		break;
/* 		case 10:
			// Datei (Verzeichnis) auswählen
			sprintf(dstPath, "%s%s", dstPath, "*.*");
			if( GetFilename("Eine Datei aus dem Zielordner auswählen", dstPath) == TRUE )
			{
				// Dateinamen abschneiden
				n = findLastToken(dstPath, '\\');
				dstPath[n+1] = '\0';	
			}	
		break; */
		case 10:
			if(subDir == FALSE)
			{
				// Kommandostring zusammenstellen ohne Unterverzeichnisse
				sprintf(command,"%s %s \"%s*%s\"%s\"%s%s\"","Exec ,", "DIR /B", srcPath, srcTyp, " >", srcPath, "ExportFiles.txt");
			}
			else
			{
				// Kommandostring zusammenstellen mit Unterverzeichnissen
				sprintf(command,"%s %s \"%s*%s\"%s\"%s%s\"","Exec ,", "DIR /B /S", srcPath, srcTyp, " >", srcPath, "ExportFiles.txt");
			}
			// Datei mit Liste von Zeichnungsnamen erstellen
			SetFuncText("Datei ExportFiles.txt erstellen");
			MegaCMD(command);
			
			// Datei zum lesen oder schreiben öffnen
			SetFuncText("Datei ExportFiles.txt öffnen");
			// String zusammenstellen 
			sprintf(command,"%s%s", dstPath, "ExportFiles.txt");
			fh1 = ML_Open(command, ML_READ | ML_TEXT | ML_DENY_RW);
			// String zusammenstellen 
			sprintf(command,"%s%s", dstPath, "ExportFiles.log");
			fh2 = ML_Open(command, ML_WRITE | ML_TEXT | ML_DENY_RW);
			
			count  = 0;
			if((fh1 != 0) && (fh2 != 0))
			{
 				SetFuncText("Datei ExportFiles.txt lesen");
				while(NULL != ML_Gets(line, sizeof(line), fh1))
				{
					count++;
					ML_Printf(fh2, "\n");
 					if(strlen(line) != NULL)
					{
						// Zeilenumbruch abschneiden
						n = findLastToken(line, '\n');
						line[n] = '\0';
						if(subDir == FALSE)
						{
							// Kommandostring zum Zeichnung Laden zusammenstellen
							sprintf(command,"%s\"%s%s\"","PrtLoad ", srcPath, line);
							sprintf(dstPath,"%s", srcPath);
						}
						else
						{
							// Kommandostring zum Zeichnung Laden zusammenstellen
							sprintf(command,"%s\"%s\"","PrtLoad ", line);
							sprintf(dstPath,"%s", line);
							// Dateinamen abschneiden
							n = findLastToken(dstPath, '\\');
							dstPath[n+1] = '\0';
						}
						//Datei aus Liste laden
						ML_Printf(fh2, "%s\n", command);
						MegaCMD(command);
						// Schleife durch alle Arbeitsblätter
						WorkingSheetLoop( SaveWorkingSheet, edta );
					} 
				} 
				ML_Close(fh1);
				ML_Printf(fh2, "\n%i Dateien gefunden", count);
				ML_Close(fh2);
				SetFuncText("Export fertig");
				//log Datei öffnen
				sprintf(command,"%s %s \"%s%s\"","Exec w,", "notepad.exe", srcPath, "ExportFiles.log");
				MegaCMD(command);
			} 
		break;
		}
	}
	while(mret != 0);
		
	free(edta);
	
	return(0);
	
}



	
	